#include <avr/io.h>
#include <avr/interrupt.h>
#include "Matrix.h"
#include "font_5x7.h"
#include "USART.h"
#include "i2c.h"
#include "Matrix3x5.h"


unsigned char Display_Array[7]={0}; //Tableau d'affichage comportant les donnes  afficher


/*---------Initialisation de l'afficheur-----------------*/

void Matrix_Init(void)
{
	Matrix_Init_Port();		// Initialisation des ports 
	Init_PWM(); 			// Initialisation du PWM sur le Timer 2 connect  l'horloge ext
	Matrix_Column (0);		// Eteinds les led de la ligne courante
	setPWM(255);			// Initialise le PWM au maximum de sa luminosit
	Matrix_Init_Timer0();	// Initialisation du Timer0 en interruption pur l'affichage.	
}


/*-----------Mise  jours des donnes  afficher en ligne-------------*/
// 7 donnes de 5 bits sont entres en paramtre et sont directements intgres dans le tableau d'affichage 
void UpdateDisplayRows (unsigned char *Data)
{
	char i;
	for(i=0;i<7;i++)
            Display_Array[i]=Data[i];//Rcupration des donnes du tableau rentr en paramtre dans le tableau d'affichage            
}


/*-----------Mise  jours des donnes  afficher en colonne-------------*/
//5 donnes de 7 bits qui sont converties et 7 donnes de 5 bits qui sont ensuite insres dans le tableau d'affichage 
void UpdateDisplayColumns (unsigned char *Data)
{
	char i,j;
	//conversion des donnees reut en parametre 5[7] -> 7[5] (conversion colomns to rows)
        TIMSK0 &= ~(1<<OCIE0A);
		for(i=0;i<7;i++){
            Display_Array[i]=0;//initialisation de la valeurs  mettre  jours
            for(j=0;j<5;j++)
                Display_Array[i]|= (((Data[4-j]&(1<<i))>>i)<<j); //mise  jours de la vleurs avec convertion
	    }
		TIMSK0 |= (1<<OCIE0A);		
}
  




/*---------------Initialisation des ports------------*/

void Matrix_Init_Port(void)
{
	// Colonnes
	DDRD|=(1<<DD2);		// Configure PD2 en sortie -> P_C5
	DDRC|=0x0F;    		// Configure PC0->3 en sortie -> P_C0  P_C4
	// Lignes
	DDRB|=0x07;			// Configure PB0->3 en sortie -> A,B et C
	DDRD|=(1<<DD3);		// Configure PD3 en sortie -> PWM
	PORTD|=(1<<PD3);	// Active la broche PWM
}



/*--------------Initialisation PWM------------------*/

// Le PWM commande le contraste de l'cran
void Init_PWM(void)
{
	DDRD|=(1<<DD3);				// Broche PWM PORTD Broche 3 en sortie
	ASSR = (1<<AS2);        	// Timer 2 connect  l'horloge ext
	// 00   A off, toggle on compare
    // 00   B off, toggle on compare
    // 00   C channel does not exist
    // 11   Low part of WGM: PWM, Fast PWM
    TCCR2A=0b00000011;
    // 0    No input noise cancelling
    // 0    Input edge select does not matter
    // 0    Not used
    // 00   Fast PWM
    // 010  Prescale 8
    TCCR2B=0b00000010;
	//initialisation de tcnt
	TCNT2=0x00;
	//Slection de la broche PD3 comme sortie PWM
	TCCR2A|=(1<<5);
    TCCR2A|=(1<<4);
}


/*--------------Initialisation Timer0------------------*/
//Sur interruption
void Matrix_Init_Timer0(void)
{
	TCCR0A |= (1<<WGM01);							//mode CTC => Comparaison
	TCCR0B |= (1<<CS00) | (0<<CS01) |(1<<CS02);		//Prdivision par 1024
	OCR0A = 40;										//valeur pour laquelle il y a interruption 
	TIMSK0 |= (1<<OCIE0A);							//lancement du timer
	sei(); 											//autorisation gnrale des interruptions
}






/*---------------Slection ligne Afficheur----------------*/
// 
// Num_Row = 0 -> Slectionne la premire ligne
// Num_Row = 1 -> Slectionne la deuxime ligne
// ...
// Num_Row = 7 -> Slectionne la septime ligne

void Matrix_Select_Row(char Num_Row)
{
	PORTB= (PORTB & 0xF8) | (Num_Row & 0x07);	// Effectue le masquage pour la slection de la ligne
}




/*-----------Commandes des leds sur une ligne--------------*/
// Data = 0x00 -> Motif = xxxxx (aucune led allume)
// Data = 0x01 -> Motif = xxxxo
// Data = 0x02 -> Motif = xxxox
// Data = 0x03 -> Motif = xxxoo
// ...
// Data = 0x1F -> Motif = ooooo (toutes les leds allumes)

void Matrix_Column (unsigned char Data) //gre les donnes en fonction de la donne rentr en paramtre
{	
	if (Data & 0x10) PORTC &= ~(1<<PC1); else PORTC |= (1<<PC1); //gestion de la premire ligne
	if (Data & 0x08) PORTC &= ~(1<<PC2); else PORTC |= (1<<PC2); //gestion de la seconde ligne
	if (Data & 0x04) PORTC &= ~(1<<PC3); else PORTC |= (1<<PC3); //gestion de la troisime ligne
	if (Data & 0x02) PORTC &= ~(1<<PC0); else PORTC |= (1<<PC0); //gestion de la quatrime ligne
	if (Data & 0x01) PORTD &= ~(1<<PD2); else PORTD |= (1<<PD2); //gestion de la cinquime ligne
}


/* Fonction d'affichage du caractre rentr en paramtre */
//Utilisation de la librairie Font5x7
//Extraction des informations de Font[]
//Mise  jours du tableau d'affichage

void Matrix_Caractere(unsigned char caract)
{
	char i=0;
	char donnee[5];
	for(i=0;i<5;i++)
	{
	donnee[i]=Font[((caract-32)*5)+i];
	}
	UpdateDisplayColumns(donnee);	
}



/*--------------Rooutine d'intrruption Timer0------------*/
//slectionne une ligne diffrentes  chaque passage et y applique la donne coreespondante du tableau d'affichage.
ISR(TIMER0_COMPA_vect)
{	
	
	static unsigned char i=0,Display[7];
	Matrix_Column(0);	//teind la ligne slectionne
	Matrix_Select_Row(i);//slectionne la ligne correspondante			
	Matrix_Column(Display[i]);//Allume les leds de la ligne slectionne grace au tableau d'affichage 	
	i++;			//|
					//|-> si la dernire ligne atteinte on recommence  la premire	
	if (i==7){for(i=0;i<7;i++) Display[i]=Display_Array[i]; i=0;}	//|
}

//-----------------------------------------------



