#include <avr/io.h>
#include <util/twi.h>
#include <avr/interrupt.h>


#include "i2c.h"
//------------variable-------------------------------------
// send/transmit buffer (outgoing data)
static unsigned char SendData[TailleData];
static unsigned char IndexEnvoyerData;	
static unsigned char *i2cSlaveTransmit;

// receive buffer (incoming data)
static unsigned char ReceiveData[TailleData];
static unsigned char IndexRecevoirData;
static unsigned char *i2cSlaveReceive;


//=========================================================


//----------------------interruption-----------------------
ISR(TWI_vect)
{
	
	unsigned char status = (TWSR & 0xF8);// status bits

	switch(status)
	{
// codes status du mode "Slave Receiver"
	case 0x60:					// 0x60: own SLA+W has been received, ACK has been returned
	case 0x68:					// 0x68: own SLA+W has been received, ACK has been returned
	case 0x70:					// 0x70:     GCA+W has been received, ACK has been returned
	case 0x78:					// 0x78:     GCA+W has been received, ACK has been returned
		// nous sommes adresss en tant qu'esclave pour crire (des donnes seront reues du matre)
		
		
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWEA));// recevez l'octet de donnes et renvoyez l'ACK
		break;
	case 0x80:					// 0x80: l'octet de donnes a t reu, ACK a t retourn
	case 0x90:					// 0x90: l'octet de donnes a t reu, ACK a t retourn
		
		i2cSlaveReceive[IndexRecevoirData]=ReceiveData[IndexRecevoirData++] = TWDR;//prendre l'octet de rception de donnes prcdemment
		if(IndexRecevoirData < TailleData)
		{
			// recevez l'octet de donnes et renvoyez l'ACK
			i2cRecevoirBit(TRUE);
			
		}
		else
		{
			// recevez l'octet de donnes et renvoyez NACK
			i2cRecevoirBit(FALSE);
			
		}
		break;
	case 0x88:					// 0x88: l'octet de donnes a t reu, NACK a t retourn
	case 0x98:					// 0x98: l'octet de donnes a t reu, NACK a t retourn
		
		i2cRecevoirBit(FALSE);// recevez l'octet de donnes et renvoyez NACK
		//TWCR=((TWCR&TWCR_MASK)|(1<<TWINT));
		break;
	case 0xA0:					// 0xA0: STOP ou REPEATED START a t reu alors qu'il est adress en tant que esclave
		
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWEA));// commutez au mode de SR avec SLA ACK
		IndexRecevoirData=0;
		// i2c receive is complete, call i2cSlaveReceive
		//if(i2cSlaveReceive) 
		break;

// Slave Transmitter
	case 0xA8:						// 0xA8: possder SLA+R a t reue, ACK a t retourne
	case 0xB0:						// 0xB0: Arbitrage perdu dans SLA+R/W comme matre ;
									//		possder SLA+R a t reue ; L'ACK a t retourn
		// nous sommes adresss en tant qu'esclave pour lire (des donnes doivent tre transmises de nouveau au matre)
		IndexEnvoyerData=0;
		//  transmettre le premier octet de donnes
	case 0xB8:					// 0xB8: l'octet de donnes a t transmis, ACK a t reu
		
		TWDR=i2cSlaveTransmit[IndexEnvoyerData];// transmettez l'octet de donnes
		if(IndexEnvoyerData < TailleData){
			
			TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWEA));// attendez l'ACK  l'octet de donnes
			// donnes de demande d'application
		}
		else
		{	
			TWCR=((TWCR&TWCR_MASK)|(1<<TWINT));// attendez NACK  l'octet de donnes
		}	
		break;
	case 0xC0:					// 0xC0: l'octet de donnes a t transmis, NACK a t reu
	case 0xC8:					// 0xC8: Le dernier octet de donnes dans TWDR a t transmis (TWEA =  0") ;
								//		ACK a t reu
		
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWEA)|(1<<TWSTO));// commutez pour ouvrir l'esclave.
		break;

// autres
	case 0xF8:					// 0xF8: Aucune information d'tat appropri

		break;
	case 0x00:					// 0x00: L'erreur sur le bus due au dbut illgal ou arrtent la condition
		
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWSTO)|(1<<TWEA));// remettez  zro le matriel interne et librez l'autobus
		break;
	}
}

//==============================================================================


//----------Initialisation de l'I2C--------------------

void i2cInit(void)
{
	TWCR=0x45;	//Slave Transmitter Mode
	TWBR=0xFF;	//
	TWSR|=0x03;	//Valeur prescaler : 64
	TWAR=ADDR; // adresse 
}

//----------------Recevoir la data---------------
void i2cRecevoirBit(unsigned char ACK)
{
	if(ACK)
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWEA));//envoie l'ACK	
	else
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT));//envoie le NACK
}


//-----------------dclaration du tableau de rception des donnes -----------------
void i2cSetSlaveReceiveHandler(unsigned char* recieveData)
{
	i2cSlaveReceive = recieveData;
}

//-----------------dclaration du tableau d'mission des donnes ------------------
void i2cSetSlaveTransmitHandler(unsigned char* sendData)
{
	i2cSlaveTransmit = sendData;
}
