#include <avr/io.h>
#include <util/twi.h>
#include <avr/interrupt.h>
#include "font_5x7.h"
#include "i2c.h"
//------------variable
volatile unsigned char TailleData=TailleDonnee; //initialise la taille des donnees
volatile unsigned char DeviceADDR_RW;
// send/transmit buffer (outgoing data)
volatile unsigned char SendData[TailleDonnee];	
volatile unsigned char IndexEnvoyerData;
// receive buffer (outgoing data)
volatile unsigned char ReceiveData[TailleDonnee];
volatile unsigned char IndexRecevoirData;

volatile unsigned char I2cState;
//=========================================================


//----------------------interruption------------------------
ISR(TWI_vect)
{
	// status bits
	unsigned char status = (TWSR & 0xF8);

	switch(status)
	{
// Master General
	case 0x08:					// 0x08: start condition
	case 0x10:					// 0x10: a repeter l'etat de dbut
		// envoye l'address de l'appareil
		i2cEnvoyeBit(DeviceADDR_RW);

		break;
	
	// Codes status du mode "Master Transmitter & Receiver"
	case 0x18:					// 0x18: Adresse esclave reconnu
	case 0x28:					// 0x28: Data reconnu
		if(IndexEnvoyerData <TailleData)
		{
			// envoye la data
			i2cEnvoyeBit( SendData[IndexEnvoyerData++] );
		}
		else
		{
			// transmit stop condition, enable SLA ACK
			// le bus libr
			I2cState = FALSE;
			i2cStop();

		}
		break;	
	case 0x58:					// 0x58: Data reu, avec rponse NACK
		// stockez l'octet final de rception de donnes
		ReceiveData[IndexRecevoirData++] = TWDR;
		// continue de transmittre la condition STOP 
	case 0x48:					// 0x48: l'addresse esclave n'est pas reconnu
	case 0x20:					// 0x20: l'addresse esclave n'est pas reconnu
	case 0x30:					// 0x30: Data n'est pas reconnu
		// transmit stop condition, enable SLA ACK
		i2cStop();
		
		// le bus libr
		I2cState = FALSE;
		break;
	case 0x38:					// 0x38: Bus arbitration lost
		// librez bus
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT));
		// le bus libr
		I2cState = FALSE;
		// librez l'autobus et transmettez le dbut quand l'autobus est libre
		//TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWSTA));
		break;

	case 0x50:					// 0x50: Data reconnu
		// stockez l'octet de rception de donnes
		ReceiveData[IndexRecevoirData++] = TWDR;		
		// Regarder si plus d'octet seront reus

	case 0x40:					// 0x40: l'addresse esclave reconnu
		if(IndexRecevoirData < (TailleData-1))
			// l'octet de donnes sera reu, rponse avec ACK (plus d'octets dans le transfert)
			i2cRecevoirBit(TRUE);
		else{
			// l'octet de donnes sera reu, rponse avec NACK (derniers octets dans le transfert)
			i2cRecevoirBit(FALSE);
			
			}
			
		break;
	// autres
	case 0xF8:					// 0xF8: Aucune information d'tat appropri

		break;
	case 0x00:					// 0x00: L'erreur sur le bus due au dbut illgal ou arrtent la condition
		// remettez  zro le matriel interne et librez l'autobus
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWSTO)|(1<<TWEA));
		// le bus libr
		I2cState = FALSE;
		break;
	}
}


//==============================================================================

//----------Initialisation--------------------

void i2cInit(void)
{
	TWCR=0x00;	//master Transmitter Mode
	TWCR |= (1<<TWIE);
	TWBR=0xFF;	//
	TWSR|=0x03;	//Valeur prescaler : 64
	TWAR=ADDR;
	sei();
}


//========================================================================

//-----------------Condition start------------------
void i2cStart(void)
{	
	TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWSTA)|(1<<TWEN));//Condition start du mode master
	loop_until_bit_is_set(TWCR, TWINT);//Attendre 
}


//-----------------Condition stop-------------
void i2cStop(void)
{
	TWCR=((TWCR & TWCR_MASK)|(1<<TWINT)|(1<<TWEN)|(1<<TWSTO));// arreter la transmission
}


//=========================================================================

//-----------------Envoyer la data-------------------
void i2cEnvoyeBit(unsigned char data)
{
	TWDR=data;
	TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWEN));	//commencer a envoyer la data
	loop_until_bit_is_set(TWCR, TWINT);				//attendre la fin de transmission
}

//----------------Recevoir la data---------------
void i2cRecevoirBit(unsigned char ACK)
{
	if(ACK)
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT)|(1<<TWEA));//envoie l'ACK	
	else
		TWCR=((TWCR&TWCR_MASK)|(1<<TWINT));//envoie le NACK
}
	

//======================================================================

//-----------------Transmission X donnees--------------
void i2cMasterSendX(unsigned char deviceAddr, unsigned char length, unsigned char *data)
{
	unsigned char i;
	
	while(I2cState);// wait for interface to be ready
	I2cState = TRUE;// set state	
	// save data
	DeviceADDR_RW = (deviceAddr & 0xFE);	// RW cleared: write operation	
	for(i=0; i<length; i++)
	SendData[i] = *data++;	//Initialise le tableau a envoyer
	IndexEnvoyerData = 0;	
	TailleData=length;	
	i2cStart();// send start condition

}

//----------------Transmission d'une donne--------------------------
void i2cMasterSend (unsigned char deviceAddr,unsigned char data)
{
	while(I2cState);// wait for interface to be ready	
	I2cState = TRUE;// set state	
	DeviceADDR_RW = (deviceAddr & 0xFE);	// RW cleared: write operation
	SendData[0] = data;
	IndexEnvoyerData = 0;
	TailleData = 1;
	i2cStart();// send start condition
}


//===========================================================================

//------------------Recevoir X donnees------------------

void i2cMasterReceiveX(unsigned char deviceAddr, unsigned char length, unsigned char* data)
{
	unsigned char i;
	
	while(I2cState);// wait for interface to be ready
	
	I2cState = TRUE;// set state
	// save data
	DeviceADDR_RW = (deviceAddr | 0x01);	// RW set: read operation
	IndexRecevoirData = 0;
	TailleData = length;
	
	i2cStart();// send start condition
	
	while(I2cState);// wait for data
	// return data

	for(i=0; i<length; i++)
		*data++ = ReceiveData[i];
}

//--------------------Recevoir une donne----------------------

void i2cMasterReceive(unsigned char deviceAddr, unsigned char data)
{
	
	while(I2cState);// wait for interface to be ready
	
	I2cState = TRUE;// set state
	// save data
	DeviceADDR_RW = (deviceAddr | 0x01);	// RW set: read operation
	IndexRecevoirData = 0;
	TailleData = 1;
	
	i2cStart();// send start condition	
	while(I2cState);// attendre la donne
	// retourne la donne
	
	data=ReceiveData[0];
}

/*-------------Envoi d'un message complet----------*/
//envoi  l'adresse le Text de longeur length
void Master_SendMessage_font(char addresse,char *Text,unsigned char length)
{
	int i;
	long y;
	for(i=0;i<length;i++)
	{ //tant que tout le message n'a pas t envoy
		i2cMasterSendX(addresse,5,&Font[((Text[i]-32)*5)]);//envoie d'un acaractre.
		for (y=0;y<(long int)10e7;y++);
	}
}

//envoi  l'adresse adresse le Text de longeur length fnction avec le mode caractr de l'esclave
void Master_SendMessage_carac(char adresse,char *Text,unsigned char length)
{
	int i;
	long y;
	for(i=0;i<length;i++)
	{ //tant que tout le message n'a pas t envoy
		i2cMasterSend(adresse,Text[i]);//envoie d'un acaractre.
		for (y=0;y<(long int)10e7;y++);
	}
}


/* 	fonction qui permet de mettre tout un text 
	traduit pour l'affichage (via font_5x7) dans un tableau
 	avec un espace entre chaque caractre 
*/
/*
	expemple : Text ={"sa"}; ==> donnee={0x48,0x54,0x54,0x54,0x20,0x0,  //s
										 0x20,0x54,0x54,0x54,0x78,0x0}; //a
*/
void Master_Text_defil(char *Text,char *donnee, unsigned char length_Text)
{

unsigned int i;
int y=0;

/*------Premire lettre du text gre-----------*/
    for(i=0;i<5;i++)
    {
        donnee[(5*y)+i]=Font[((Text[y]-32)*5)+i]; //rcupration des dones
    }
    y++;
    donnee[(6*y)-1]=0; // cration d'un espace


/*-------reste du text grer---------------------*/
    while(y < length_Text - 1) // tant que toutes les lettres n'ont pas t traites.
    {
        for(i=0;i<=5;i++)
        {
			if(i==5)
            	donnee[(6*y)+i]=0; // cration de l'espace aprs la lettre 
            else
            	donnee[(6*y)+i]=Font[((Text[y]-32)*5)+i]; //rcupration des donnes de la lettre en cours
        }
        y++; // lettre suivante
    }

}
/*-----------Fonction de dfilement sur un afficheur-----------*/
void Master_defil1(char adresse,char *donnee,unsigned char length)
{
	int i;
	int y=0;
	char z;
	char donnee_5[5]={0};
	for (i=0;i<length;i++) // affichage de la taille du message.
	{
		for(z=0;z<5;z++)
		{
			donnee_5[z]=donnee[i+z]; //rcupration des donnes  afficher
		}

		i2cMasterSendX(adresse,5,donnee_5); //envoie des donnes  afficher 
		for (y=0;y<(3*(long int)10e3);y++);
	}
}
